from .xaisuiteFoundation import*
from .dataHandler import DataProcessor

class ModelTrainer:
  '''
  Class to train an explainable machine learning model.
  
  :param Any model: The string name of the model, the function returning the model, or the model Any itself. The model function must have fit() and predict() functions. A score() function is optional.
  :param DataProcessor withData: The data that will be used to train and test the model
  :param str, optional taskType: The type of task that the model performs. By default, "Tabular". Other options are "Vision" and "NLP"
  :param str, optional task: The task that the model performs. By default, "regression". Other option is "classification"
  :param list, dict, optional explainers: A list of explainer names or, if specific parameters need to be passed to the explainers, a dict that contains the explainer names and explainer arguments. Ex. explainers = ["lime", "shap", "mace"] or explainers = {"lime": {"kernel_width": 3}, "shap": {"nsamples": 100}, "mace": None}
  '''

  def __init__(self, model:Any, withData: DataProcessor, taskType:str = "Tabular", task:str = "regression", explainers:Union[list, dict] = None, **modelArgs):
    '''
    Class constructor
    '''
    self.withData = withData
    tempModel = model
    if isinstance(model, str):
      try:
        tempModel = eval(model + "(**modelArgs)")
      except:
        if model in linkModels.keys():
          exec("from " + linkModel.get(model) + " import*")
          tempModel = eval(model + "(**modelArgs)")
    elif isinstance(model, Callable):
        tempModel = model(**modelArgs)

    model = tempModel
    self.model = model

    try:
        model.fit(withData.X_train, withData.y_train)
    except Exception as e:
        print("Model could not be fit to data: \n" + str(e))

    score = 0

    try:
        score = model.score(withData.X_test, withData.y_test)
    except:
        score = r2_score([model.predict(x) for x in withData.X_test], withData.y_test)

    print("Model score is " + str(score))

    #Model has been trained by this point. Now for the explanations

    explainer_names = explainers if isinstance(explainers, list) else explainers.keys()

    self.explainer = eval(taskType + "Explainer(explainers = explainer_names, mode = task, data = withData.loader.wrappedData, model = model, preprocess = withData.processor.transform, params = explainers if isinstance(explainers, dict) else None)")

  def getExplanationsFor(self, testIndex:Union[int, list] = None, feature_values:dict = None) -> dict:
    '''
    Function to get the local explanations for a particular testing instance. 
    
    :param Union[int, list], optional testIndex: The indices of the testing data for which to fetch local explanations. If empty, local explanations for all instances are returned. If None, `feature_values` is used.
    :param dict, optional feature_values: The values of the features corresponding to a particular index. If None, `testIndex` is used. 
    :returns dict explanations: The requested explanations
    :raises ValueError: If neither testIndex or feature_values is passed
    '''
    if testIndex is None and feature_values is None:
        raise ValueError("One of testIndex or feature_values must be provided.")

    if testIndex is not None and feature_values is not None:
        print("Both testIndex and feature_values were provided. Using testIndex.")

  

    if isinstance(testIndex, list) and len(testIndex) == 0:
        return self.explainer.explain(self.withData.processor.invert(self.withData.processedData.X_test))
    elif isinstance(testIndex, list) and len(testIndex) >0:
        return self.explainer.explain(self.withData.processor.invert(numpy.array([self.withData.processedData.X_test[i] for i in testIndex])))
    elif isinstance(testIndex, int):
        return self.explainer.explain(self.withData.processor.invert(numpy.array(self.withData.processedData.X_test[testIndex])))

  #By this point, we are aware that we are dealing with feature_values and testIndex must be None. 

    assert(testIndex is None), "Something went wrong. Please try again."

  #First, we fetch all the feature data for the dataset
    data = self.withData.loader.X
  
  #Next, we create a string that resolves into a boolean expression when evaluated and will be a search query constructed from the provided feature_values
    query = []
    for feature, value in feature_values.items():
        query.append("df['" + feature + "'] == " + value)
    queryString = " and ".join(query)
    return self.explainer.explain(data.loc[eval(queryString)])

  def getSummaryExplanations(self) -> dict:
    '''
    Returns global explanations
    
    :returns dict explanations: The requested global explanations
    '''
    return self.explainer.explain_global()



from .xaisuiteFoundation import*

#Class XAIData starts here
class XAIData:
  '''
  Class to represent data. 

  :param numpy.ndarray X_train: The training features
  :param numpy.ndarray X_test: The testing features
  :param numpy.ndarray y_train: The training targets
  :param numpy.ndarray y_test: The testing targets
  '''
  def __init__(self, X_train, X_test, y_train, y_test):
    self.X_train = X_train
    self.X_test = X_test
    self.y_train = y_train
    self.y_test = y_test
    self.trainingData = (X_train, y_train)
    self.testingData = (X_test, y_test)

#Class Data ends here

#Class DataLoader starts here

class DataLoader:
  '''
  Class that loads data from a given source

  :param Union[str, Callable, numpy.ndarray, pd.DataFrame, tuple] data: The data identifier, a function that returns the data, or the data itself in the form of a numpy array, pandas DataFrame, or tuple
  :param str, optional source: The source of the data. Either "auto", "system", "preloaded", "generated", or "url". If "auto", the source will be inferred based on `data`. By default, "auto"
  :param str, optional type: The type of data. Either "Tabular", "Image", or "Text". By default, "Tabular". If "Text" or "Image", only one feature is allowed. 
  :param Union[str, list], optional variable_names: The variables in the dataset excluding `cut`, in the order that they appear in the data. By default, set to "auto" and inferred. 
  :param Union[str, int], optional target_names: The target variable(s). By default, set to "auto" and inferred
  :param Union[str, list], optional cut: Variables to drop from the data. By default, None. 
  :param Union[str, list], optional categorical: If type == "Tabular", variables that contain categorical data.
  :param `**dataGenerationArgs`: Additional arguments to pass in if `data` is Callable. 
  :raises ValueError: if data cannot be resolved or if invalid arguments are passed. 
  '''
  def __init__(self, data:Union[str, Callable, numpy.ndarray, pd.DataFrame, tuple], source:str = "auto", type:str = "Tabular", variable_names:Union[str, list] = "auto", target_names:Union[str, int] = "auto", cut:Union[str, list] = None, categorical:Union[str, list] = None, **dataGenerationArgs):
    '''
    Class constructor
    '''
    #Initially, we set the data content to None. self.content is meant to be a pd.DataFrame
    self.type = type
    self.content = None
    #If data is a function that returns the dataset or a string representation of such a function, we reassign data to the return value of this function. 
    if isinstance(data, Callable):
        data = data(**dataGenerationArgs) if dataGenerationArgs is not None else data()


    #If data is still a function, that means that the original value provided for data did not return a valid data value. 
    if isinstance(data, Callable):
        raise ValueError("Callable passed to DataLoader returns another Callable instead of a string, numpy.ndarray, pd.DataFrame, or tuple.")

    #If data is a DataFrame, the work is already done for us. 
    if isinstance(data, pd.DataFrame):
        self.content = data
    #Simple case of passing the provided numpy array directly to the DataFrame constructor
    elif isinstance(data, numpy.ndarray):
        self.content = pd.DataFrame(data)
    #We assume the tuple is of the form (X, y), where X and y are individually a valid data type, such as numpy.ndarray or pandas.DataFrame
    elif isinstance(data, tuple):
        self.content = pd.DataFrame(data[0])
        self.content["target"] = data[1]
    #If the data is a string, there are multiple possibilities that we need to check. 
    elif isinstance(data, str):
        match source:
            case "system":
                self.initializeDataFromSystem(data) #The source is system, so we search for a file
            case "preloaded":
                self.initializeDataFromPreloaded(data) #The source is preloaded, so we search for the data in a preloaded dictionary
            case "generated":
                self.initializeDataFromGenerated(data, **dataGenerationArgs) if dataGenerationArgs is not None else self.initializeDataFromGenerated(data, None) #The source is generated, and the string is the name of a function, so we pass data and the variables to pass to the data-generating function
            case "url":
                self.initializeDataFromUrl(data) #The source is a url, so we try to get the data from the url
            case "auto": #Here, the user is either unaware of where to look or just lazy :) Anyway, we need to do some hard work. We need to try each data loading method and see what sticks.
                try:
                    self.initializeDataFromSystem(data) 
                except:
                    try:
                      self.initializeDataFromPreloaded(data)
                    except:
                        try:
                            self.initializeDataFromGenerated(data, **dataGenerationArgs) if dataGenerationArgs is not None else self.initializeDataFromGenerated(data, None)
                        except:
                            try:
                                  self.initializeDataFromUrl(data)
                            except:
                                  raise ValueError("Data is not valid. Please make sure your data string is not misspelt and exists.") #This means we could not find the data anywhere.
                  
    assert isinstance(self.content, pd.DataFrame), "A problem occurred with the data loading. If the problem persists, file an issue at github.com/11301858/XAISuite" #By this time, data should definitely be a dataframe. If it is not, something has gone horribly wrong. 

    if cut is not None:
        self.content.drop(cut, axis = 1, index = None, columns = None, level = None, inplace = True, errors = 'raise') #Remove the cut variable (if it exists)

    #Now, we should have a dataframe with the features and the target. If the data type is image or text, there should be only one feature

    if (type == "Image" or type == "Text") and len(variable_names) < 2:
        return ValueError("For Image and Text data, there must be only one feature.")

    #Make sure that variable_names are the same length as the number of columns in the data provided. 

    if variable_names != "auto" and (len(variable_names) != len(self.content.columns)):
        raise ValueError("The length of variable_names is incompatible with the data provided.")

    #Make sure that the target variable, if provided, is in the provided variable_names, if provided

    if target_names != "auto" and variable_names != "auto" and target_names not in variable_names:
        raise ValueError("Custom target variable name not found in variable_names.")

    #If the target variable is auto, we determine what the value of the target variable will be first:
    if target_names == "auto" and "target" in self.content.columns:
        target_names = "target"
    elif target_names == "auto" and "target" not in self.content.columns:
        target_names = self.content.columns[-1]

    #If variable_names is auto and target is not auto, there is nothing that needs to be done

    

    #If variable_names is not auto and target is auto, we need to make sure that variable_names does not override the auto target value. 
    if variable_names != "auto" and target_names == "auto":
        target_names = variable_names[-1] if not isinstance(variable_names, str) else variable_names
    elif variable_names != "auto" and target_names == "target":
        target_names = variable_names[self.content.columns.get_loc("target")]

    #Now we set the variable_names.
    if variable_names != "auto": #Do nothing if the variable_names are set to auto
        self.content.columns = variable_names

    if categorical is not None and categorical not in variable_names:
        raise ValueError("Categorical variables provided are incompatible with variable_names.")

    #Now we split the data to make it easier to handle:

    self.y = pd.DataFrame(self.content[target_names])
    self.X = pd.DataFrame(self.content.drop(target_names, axis=1))
    self.target = target_names

    #Now we're ready to finalize creating the data object
    self.wrappedData = None
    if type == "Tabular":
        self.wrappedData = Tabular(data = self.content, categorical_columns = categorical, target_column = self.target)
    elif type == "Image":
        self.wrappedData = Image(data = self.X.to_numpy(), batched = True)
    elif type == "Text":
        self.wrappedData = Text(data = self.X.values.reshape(-1,).tolist())
    return

  def initializeDataFromSystem(self, id:str):
    '''
    Initializes data from user's system
  
    :param str id: The data file path
    :raises ValueError: if provided file path is not found or is not a file
    '''
    if not os.path.isfile(id):
        raise ValueError("Given data id is not a file path.")
    
    df = pd.read_csv(id)
    self.content = df
    return
    
  
  def initializeDataFromPreloaded(self, id:str):
    '''
    Initializes data from preloaded sklearn datasets
  
    :param str id: The name of the preloaded dataset
    :raises ValueError: if provided preloaded data name is not found
    '''
    if id in acceptedDataIDs:
        data = eval(acceptedDataIDs.get(id) + "(return_X_y=True)")
        self.content = pd.DataFrame(data[0])
        self.content["target"] = data[1]
    else:
        raise ValueError("Not an accepted preloaded data id. Available preloaded data ids are: \n" + acceptedDataIDs.keys() + "\n")
  
    return 

  def initializeDataFromGenerated(self, id:str, **generateArgs):
    '''
    Initializes data from string commands
  
    :param str id: The string generation command
    :param `**generateArgs`: Arguments to pass to the function represented by `id`
    '''
    data = eval(id + "(**generateArgs)") if generateArgs is not None else eval(id + "()")
    if isinstance(data, tuple): #We take care of the case that the generator function returns a tuple
        temp = pd.DataFrame(data[0])
        temp["target"] = data[1]
        self.content = temp
    else:
        self.content = pd.DataFrame(data)

    return  
        
  
  def initializeDataFromUrl(self, id:str):
    '''
    Initializes data from URL
  
    :param str id: The url of the data
    '''
    df = pd.read_csv(id)
    self.content = df
    return
    
#Class DataLoader ends here

#Class DataProcessor starts

class DataProcessor:
  '''
  Class that processes data
  
  :param DataLoader forDataLoader: The dataloader that will be associated with this processor. 
  :param float, optional test_size: The proportion of data that will be used to test and score the machine learning model. By default, 0.2
  :param Any, optional processor: The data processer, either a string function, or an Object with fit() and transform() methods.
  :param `**processorArgs`: Arguments to be passed into the processor. If the argument is a function, like a component of a composite processor, pass it in as shown in this example: DataProcessor(..., target_transform = "component: KBins(n_bins = 5)", ratio = 0.1)
  '''

  def __init__(self, forDataLoader:DataLoader, test_size:float = 0.2, processor:Any = None, **processorArgs):
    self.processor = processor
    self.processedX = None
    self.processedy = None
    self.loader = forDataLoader
    target_transform = None

    if processor is None: #Get default processor if processor is not provided
        match forDataLoader.type:
            case "Tabular":
                  processor = "TabularTransform"
            case "Image":
                  processor = "Scale"
            case "Text":
                  processor = "Tfidf"
    
    tempProcessorArgs = None

    if processorArgs is not None: #This is if the user passes in a component transformer as a string representation of a function
        for variable in processorArgs.items():
            if isinstance(variable[1], str) and "component:" in variable[1].replace(" ", ""):
                function_string = (variable[1].replace(" ", "")).split(":", 1)[1]
                if not function_string.endswith(")"):
                    function_string = function_string + "()"
                actualValue = eval(function_string)
                tempProcessorArgs.update((variable[0], actualValue))
            else:
                tempProcessorArgs.update(variable)
    
    processorArgs = tempProcessorArgs
    
    if forDataLoader.type != "Tabular" and processorArgs.get("target_transform") is not None:
        target_transform = processorArgs.get("target_transform")
        processorArgs.pop("target_transform")
        target_transform.fit(forDataLoader.y)
        self.processedy = target_transform.transform(forDataLoader.y)
      
        
    if isinstance(processor, str):
        self.processor = eval(processor + "(**processorArgs)") if processorArgs is not None else eval(processor + "()")
    elif isinstance(processor, Callable):
        self.processor = processor(**processorArgs) if processorArgs is not None else processor()

    self.processor.fit(forDataLoader.wrappedData)
    processedData = self.processor.transform(forDataLoader.wrappedData)
    if forDataLoader.type != "Tabular":
            self.processedX = processedData
    else:
        tempProcessedData = pd.DataFrame(processedData, columns = forDataLoader.content.columns)
        self.processedy = tempProcessedData[forDataLoader.target].to_numpy()
        self.processedX = tempProcessedData.drop(forDataLoader.target, axis=1).to_numpy()

    self.X_train, self.X_test, self.y_train, self.y_test = train_test_split(self.processedX, self.processedy, test_size = test_size)
    self.processedData = XAIData(X_train = self.X_train, X_test = self.X_test, y_train = self.y_train, y_test = self.y_test)
      
#Class DataProcessorEnds



from .xaisuiteFoundation import*
import collections

class InsightGenerator:
  '''
  Class to generate insights based on explanation results. This was first released in version 2.0.0 and is a work in progress.

  :param collections.OrderedDict explanations: Local explanation results, as generated by the `ModelTrainer.getExplanationsFor()` function
  '''
  def __init__(self, explanations:collections.OrderedDict):
    self.explanations = explanations
    self.explainers = list(self.explanations.keys())
    self.explainers.remove('predict')
    self.num_instances = len(self.explanations.get(self.explainers[0] if len(self.explainers) != 0 else "No local explanations").get_explanations())

  def calculateExplainerSimilarity(self, explainer1:str, explainer2:str) -> float:
    '''
    Calculates explainer similarity based on the average Shreyan distance. 

    :param str explainer1: The name of the first explainer. Ex. "shap", "lime"
    :param str explainer2: The name of the second explainer
    '''

    sum = 0
    for i in range (0, self.num_instances):
      sum += self.getShreyanDistance(self.explanations.get(explainer1).get_explanations()[i].get("features"), self.explanations.get(explainer2).get_explanations()[i].get("features"))
    return sum / self.num_instances
    

  def getShreyanDistance(self, vec1:list, vec2:list) -> float:
    '''
    Calculate the distance between two ordered vectors. 1 is the max distance, 0 is no distance

    :param list vec1: The pattern vector
    :param list vec2: The disorder vector
    :returns float shreyan_distance: The Shreyan distance between the two ordered vectors
    '''

    assert (len(vec1) == len(vec2)), "Cannot find the Shreyan distance when ordered vectors are of different lengths."

    x = len(vec1)
    max = self.__sigma(0, int((x/2) - 1), lambda i: ((x - 2*i - 1) * (2*x - 4*i - 1) / (x**2))) #This is the maximum possible Shreyan distance between two ordered vectors of size x

    enumerated_elements = {index + 1:value for index, value in enumerate(vec1)}

    key_list = list(enumerated_elements.keys())
    val_list = list(enumerated_elements.values())
 
    enumerated_vec2 = []
    for element in vec2:
      enumerated_vec2.append(key_list[val_list.index(element)])

    shreyan_distance = 0.0

    for j in range(1, x+1):
      shreyan_distance += (x - j) * abs(j - enumerated_vec2[j])
    
    shreyan_distance /= (x**2)
    shreyan_distance /= max
    shreyan_distance = 1 - shreyan_distance
    return shreyan_distance    

  def __sigma(self, start, end, expression):
    return sum(expression(i) for i in range(start, end+1))

  

    
    
    
      




#!/usr/bin/env python
# coding: utf-8

#(c) Shreyan Mitra
from tkinter import*
from tkinter import filedialog
import pkg_resources
pkg_resources.require("XAISuite==1.0.8")
from xaisuite import*
import pandas as pd
import os
import warnings
import matplotlib.pyplot as plt
import requests
from PIL import Image, ImageTk

warnings.filterwarnings("ignore")

class ABC(Frame):
    def __init__(self,parent=None):
        Frame.__init__(self,parent)
        self.parent = parent
        self.pack()
        self.make_widgets()
        

    def make_widgets(self):
        # don't assume that self.parent is a root window.
        # instead, call `winfo_toplevel to get the root window
        self.winfo_toplevel().title("XAISuite: Training and Explanation Generation Utilities for Machine Learning Models")
        

        # this adds something to the frame, otherwise the default
        # size of the window will be very small
        
        im = Image.open(requests.get("https://user-images.githubusercontent.com/66180831/209478341-a1b4d80b-dbcb-448c-a3e0-109e27590ec5.png", stream=True).raw)
        ph = ImageTk.PhotoImage(im)
        self.logo = ph
        self.values = []
        panel = Label(self, image = self.logo)
        panel.pack(side = "top", fill = "both", expand = "yes")
        welcome = Label(self, text = "Welcome! Please choose one of the following options:", font = 'Helvetica 18 bold')
        welcome.pack(side="top", fill="x")
        button1 = Button(self, text = 'Train and Explain Model', bd = '5',
                          command = self.trainandexplaindialogue)
        button1.pack()
        button2 = Button(self, text = 'Analyze Explanatory Methods', bd = '5',
                          command = self.analyzeexplainersdialogue)
        button2.pack()
        
        
    
    def analyzeexplainersdialogue(self):
        # Toplevel object which will
        # be treated as a new window
        analyzeexplainers = Toplevel()

        # sets the title of the
        # Toplevel widget
        analyzeexplainers.title("New Window")

        # sets the geometry of toplevel
        analyzeexplainers.geometry("200x200")
        
        Label(analyzeexplainers,
              text ="Analyze Explanations", font = 'Helvetica 18 bold').pack()

        
        def openFile():
            self.file_selected = filedialog.askopenfilenames(initialdir=os.getcwd(), title="Select Files for XAISuite", filetypes=[("CSV files", ".csv")])
            filePath.set(self.file_selected)
       
        filePath = StringVar(analyzeexplainers)
        filePath.set("Choose explainer files.")
        Button(analyzeexplainers, text="Choose your data file", command=openFile).pack()
        entry1 = Entry(analyzeexplainers, textvariable=filePath, width=50)
        entry1.pack()
        
            
        
        def core():
            try:
                print("Selected files are: " + str(list(self.file_selected)))
                compare_explanations(list(self.file_selected))
                
            except Exception as e:
                print("Comparison failed: " + str(e))
        
        Button(analyzeexplainers, text="SUBMIT", command= core).pack()
    
    
    def trainandexplaindialogue(self):
        # Toplevel object which will
        # be treated as a new window
        trainandexplain = Toplevel()

        # sets the title of the
        # Toplevel widget
        trainandexplain.title("New Window")

        # sets the geometry of toplevel
        trainandexplain.geometry("200x200")
        
        acceptedModels = {"SVC": "sklearn.svm", "NuSVC": "sklearn.svm", "LinearSVC": "sklearn.svm", "SVR": "sklearn.svm", "NuSVR": "sklearn.svm", "LinearSVR": "sklearn.svm", 
                 "AdaBoostClassifier": "sklearn.ensemble", "AdaBoostRegressor": "sklearn.ensemble", "BaggingClassifier": "sklearn.ensemble", "BaggingRegressor": "sklearn.ensemble",
                 "ExtraTreesClassifier": "sklearn.ensemble", "ExtraTreesRegressor": "sklearn.ensemble", 
                 "GradientBoostingClassifier": "sklearn.ensemble", "GradientBoostingRegressor": "sklearn.ensemble",
                 "RandomForestClassifier": "sklearn.ensemble", "RandomForestRegressor": "sklearn.ensemble",
                 "StackingClassifier": "sklearn.ensemble", "StackingRegressor": "sklearn.ensemble",
                 "VotingClassifier": "sklearn.ensemble", "VotingRegressor": "sklearn.ensemble",
                 "HistGradientBoostingClassifier": "sklearn.ensemble", "HistGradientBoostingRegressor": "sklearn.ensemble",
                 "GaussianProcessClassifier": "sklearn.gaussian_process", "GaussianProcessRegressor": "sklearn.gaussian_process", 
                 "IsotonicRegression": "sklearn.isotonic", "KernelRidge": "sklearn.kernel_ridge", 
                 "LogisticRegression": "sklearn.linear_model", "LogisticRegressionCV": "sklearn.linear_model",
                 "PassiveAgressiveClassifier": "sklearn.linear_model", "Perceptron": "sklearn.linear_model", 
                  "RidgeClassifier": "sklearn.linear_model", "RidgeClassifierCV": "sklearn.linear_model",
                  "SGDClassifier": "sklearn.linear_model", "SGDOneClassSVM": "sklearn.linear_model", 
                  "LinearRegression": "sklearn.linear_model", "Ridge": "sklearn.linear_model", 
                  "RidgeCV": "sklearn.linear_model", "SGDRegressor": "sklearn.linear_model",
                  "ElasticNet": "sklearn.linear_model", "ElasticNetCV": "sklearn.linear_model",
                  "Lars": "sklearn.linear_model", "LarsCV": "sklearn.linear_model", 
                  "Lasso": "sklearn.linear_model", "LassoCV": "sklearn.linear_model",
                  "LassoLars": "sklearn.linear_model", "LassoLarsCV": "sklearn.linear_model",
                  "LassoLarsIC": "sklearn.linear_model", "OrthogonalMatchingPursuit": "sklearn.linear_model",
                  "OrthogonalMatchingPursuitCV": "sklearn.linear_model", "ARDRegression": "sklearn.linear_model",
                  "BayesianRidge": "sklearn.linear_model", "MultiTaskElasticNet": "sklearn.linear_model", 
                  "MultiTaskElasticNetCV": "sklearn.linear_model", "MultiTaskLasso": "sklearn.linear_model",
                  "MultiTaskLassoCV": "sklearn.linear_model", "HuberRegressor": "sklearn.linear_model",
                  "QuantileRegressor": "sklearn.linear_model", "RANSACRegressor": "sklearn.linear_model",
                  "TheilSenRegressor": "sklearn.linear_model", "PoissonRegressor": "sklearn.linear_model",
                  "TweedieRegressor": "sklearn.linear_model", "GammaRegressor": "sklearn.linear_model", 
                  "PassiveAggressiveRegressor": "sklearn.linear_model", "BayesianGaussianMixture": "sklearn.mixture",
                  "GaussianMixture": "sklearn.mixture", 
                  "OneVsOneClassifier": "sklearn.multiclass", "OneVsRestClassifier": "sklearn.multiclass", 
                  "OutputCodeClassifier": "sklearn.multiclass", "ClassifierChain": "sklearn.multioutput", 
                   "RegressorChain": "sklearn.multioutput",  "MultiOutputRegressor": "sklearn.multioutput",
                   "MultiOutputClassifier": "sklearn.multioutput", "BernoulliNB": "sklearn.naive_bayes", 
                  "CategoricalNB": "sklearn.naive_bayes", "ComplementNB": "sklearn.naive_bayes", 
                  "GaussianNB": "sklearn.naive_bayes", "MultinomialNB": "sklearn.naive_bayes", 
                  "KNeighborsClassifier": "sklearn.neighbors", "KNeighborsRegressor": "sklearn.neighbors", 
                  "BernoulliRBM": "sklearn.neural_network", "MLPClassifier": "sklearn.neural_network", "MLPRegressor": "sklearn.neural_network",  
                  "DecisionTreeClassifier": "sklearn.tree", "DecisionTreeRegressor": "sklearn.tree",
                  "ExtraTreeClassifier": "sklearn.tree", "ExtraTreeRegressor": "sklearn.tree"
                 }

        # A Label widget to show in toplevel
        Label(trainandexplain,
              text ="Train and Explain Dialogue", font = 'Helvetica 18 bold').pack()
        
        model = StringVar(trainandexplain)
        model.set("Choose a Model")
        OptionMenu(trainandexplain, model, *acceptedModels.keys()).pack()
        
        def openFile():
            file_selected = filedialog.askopenfilename(title="Select File for XAISuite", filetypes=[("CSV files", ".csv")])
            filePath.set(file_selected)
       
        filePath = StringVar(trainandexplain)
        filePath.set("Choose file above or enter sklearn dataset.")
        target = StringVar(trainandexplain)
        cut = StringVar(trainandexplain)
        index = StringVar(trainandexplain)
        Button(trainandexplain, text="Choose your data file", command=openFile).pack()
        entry1 = Entry(trainandexplain, textvariable=filePath, width=50)
        entry1.pack()
        
       
        target.set("Choose a target variable")
        targetmenu = OptionMenu(trainandexplain, target, ["Placeholder"])
        targetmenu.pack()
        
        cut.set("Enter a cut variable (Optional)")
        cutmenu = OptionMenu(trainandexplain, cut, ["Placeholder"])
        cutmenu.pack()
        
        
        def update_options(*args):
            df = pd.read_csv(filePath.get())
            columns = list(df)
            
            

            options1 = targetmenu['menu']
            options1.delete(0, 'end')
            
            options2 = cutmenu['menu']
            options2.delete(0, 'end')
            

            for variable in columns:
                options1.add_command(label=variable, command=lambda choice=variable: target.set(choice))
                options2.add_command(label=variable, command=lambda choice=variable: cut.set(choice))
        
        filePath.trace('w', update_options)
        
        
        
        
        
        
        Label(trainandexplain,
              text ="Choose Explainers").pack()
        explanations = Listbox(trainandexplain, selectmode = "multiple")
  
        # Widget expands horizontally and
        # vertically by assigning both to 
        # fill option
        explanations.pack()

        # Taking a list 'x' with the items 
        # as explanatory methods
        x = ["shap", "lime", "pdp", "ale"]

        for each_item in range(len(x)):

            explanations.insert(END, x[each_item])

            # coloring alternative lines of listbox
            explanations.itemconfig(each_item,
                     bg = "white")
        
        
        def core():
            if (model.get() == "Choose a Model" or model.get() == "Model is Required"):
                model.set("Model is Required")
                return
            if (len(filePath.get()) == 0 or filePath.get() == "Data file is required." or filePath.get() == "Choose file above or enter sklearn dataset."):
                filePath.set("Data file is required.")
                return
            if (target.get() == "Choose a target variable." or target.get() == "Target Variable Required"):
                target.set("Target Variable Required")
                return
             
            if (cut.get() == "Enter a cut variable."):
                try:
                    train_and_explainModel(model.get(), load_data_CSV(filePath.get(), target.get()), x_ai = self.values) 
                except FileNotFoundError:
                    train_and_explainModel(model.get(), load_data_sklearn(filePath.get(), target.get()), x_ai = self.values) 
                except Exception as e:
                    print("Something went wrong. Check your inputs. " + str(e))
            else:
                try:
                    train_and_explainModel(model.get(), load_data_CSV(filePath.get(), target.get(), cut.get()), x_ai = self.values)
                except FileNotFoundError:
                    train_and_explainModel(model.get(), load_data_sklearn(filePath.get(), target.get()), x_ai = self.values) 
                except Exception as e:
                    print("Something went wrong. Check your inputs. " + str(e))
                    
        
        def callback(event):
            self.values = [event.widget.get(idx) for idx in event.widget.curselection()]
            
        
        Button(trainandexplain, text="SUBMIT", command= core).pack()
            
        
        
        explanations.bind('<<ListboxSelect>>', callback)
        
        
       
        
       
    

        
        

root = Tk()
root.attributes('-fullscreen', True)
abc = ABC(root)
root.mainloop()



#For creating package
from pkg_resources import get_distribution, DistributionNotFound

try:
    dist = get_distribution("XAISuite")
except DistributionNotFound:
    __version__ = "Please install XAISuite with setup.py"
else:
    __version__ = dist.version

#Package Modules
from .dataHandler import*
from .explainableModel import*
from .insightGenerator import*
from .xaisuiteFoundation import*
from models import*
from explainers import*

